/////////////////////////////////////////////////////////////////////////
///
///	\file	MediaPlayer.cpp
///	\author	Rob Bateman
///	\date	09-03-2003
///	\brief	This file defines a set of functions that hide away DirectAudio
///			behind a nice C interface. These set of functions allow you
///			to load a number of media files (wav, midi etc) for sound
///			effects and background music (midi files only though!).
///
///	\note	
///			I assume that this supports all native media types in windows.
///			(ie, divx will work if the codecs are installed on the system)
///
///
///			Usage
///			-----
///
///			You must ensure that you call MediaInit() before you call any
///			other media function. 
/// 
///			When you have finished with DirectAudio, call MediaCleanup().
/// 
///			In order to load and play a media file, you should first call
///			MediaLoad() specifying the file you wish to load. This function
///			will return to you an ID number with which you can refer to 
///			the sound. For example
///
///
///				// load the sound "SomeSound.wav" and store it's id num in "SoundID"
///				unsigned int SoundID = MediaLoad("SomeSound.wav");
///
/// 
///			you can then start and stop playback using MediaPlay() and 
///			MediaStop(). ie, 
/// 
///
///				// this starts playback of the wave file
///				MediaPlay( SoundID );
///
///			
///			You can also make use of the following sets of functions to 
///			change properties of the sound playback :
///
///
///			File Management Controls
///			------------------------
///
///			unsigned int MediaLoad ( char filename[] )	
///			void		 MediaDelete ( unsigned int id );			
///			void		 MediaDeleteAll ( );
///	
///
///			Volume Controls		-	valid values -10000 -> 0
///			---------------
///
///			void		MediaSetVolume ( unsigned int id, long balance )		sets the current volume level for the sound
///			long		MediaGetVolume ( unsigned int id )						gets current volume level of the sound
///
///
///			Balance Controls	-	valid values -10000 -> 10000
///			----------------
///
///			void		MediaSetBalance ( unsigned int id, long balance )		sets the current balance amount for the sound
///			long		MediaGetBalance ( unsigned int id )						gets current balance amount of the sound
///
///
///			Frequency Controls	-	valid values 100 -> 100000
///			------------------
///
///			void		MediaSetFrequency ( unsigned int id, long frequency )	sets the current pitch of the sound
///			long		MediaGetFrequency ( unsigned int id )					gets the current pitch of the sound
///
///
///
///			3D Sound Controls [Sound Position Controls]
///			-------------------------------------------
///
///			void		MediaGetConeAngles ( unsigned int id, long* pInsideConeAngle, long* pOutsideConeAngle );
///
///			void		MediaGetConeOrientation ( unsigned int id, float orient[] );
///
///			long		MediaGetConeOutsideVolume ( unsigned int id );
///
///			void		MediaGetPosition ( unsigned int id, float position[] );
///
///			void		MediaGetVelocity ( unsigned int id, float velocity[] );
///
///			float		MediaGetMaxDistance ( unsigned int id );
///		
///			float		MediaGetMinDistance ( unsigned int id );
///
///			
///			void		MediaSetConeAngles ( unsigned int id, long InsideConeAngle, long OutsideConeAngle );
///
///			void		MediaSetConeOrientation ( unsigned int id, float orient[] );
///
///			void		MediaSetConeOutsideVolume ( unsigned int id, long volume );
///
///			void		MediaSetPosition ( unsigned int id, float position[] );
///
///			void		MediaSetVelocity ( unsigned int id, float velocity[] );
///
///			void		MediaSetMaxDistance ( unsigned int id, float dist );
///
///			void		MediaSetMinDistance ( unsigned int id, float dist );
///			
///
///
///			3D Sound Controls [Listener Position Controls]
///			----------------------------------------------
///
///			float		MediaGetDopplerFactor ( );
///
///			void		MediaGetOrientation ( float frontVec[], float upVec[] );
///
///			void		MediaGetPosition ( float position[] );
///
///			float		MediaGetRollOffFactor ( );
///
///			void		MediaGetVelocity ( float velocity[] );
///
///
///			void		MediaSetDopplerFactor ( float val );
///
///			void		MediaSetOrientation ( float frontVec[], float upVec[] );
///
///			void		MediaSetPosition ( float position[] );
///
///			void		MediaSetRollOffFactor ( float val );
///
///			void		MediaSetVelocity (  float velocity[] );
///
///
///
///			3D Sound Update Func
///			--------------------
///
///			void		MediaUpdate3D ();
///
///
///
///			3D Sound Controls	[ READ ALL OF THIS BEFORE USE ]
///			====================================================
///
///
///			Essentually the 3D sound controls are split into 2 catagories : 
///
///			1. Positioning The Listener (Ear if you will) in 3D space
///			2. Positioning The Sound in 3D space
///
///			There are three main effects available as part of 3D sound are
///
///			1. Position - a sound may be placed far away or nearby. The volume of 
///					that sound will depend on this distance between the listener
///					and the sound source location. 
///
///			2. Orientation - a sound may occur behind or infront of you 
///					depending on the direction of the listener,
///
///			3. Doppler Effect - the doppler effect is the change of a sounds
///					frequency which is caused due to the sound source or listener
///					travelling at speed. (ie, the change in pitch of a police siren
///					as it speeds past you ).
///
///			
///			All of these effects are calculated automatically and only require you
///			to specify things like the position, velocity and orientation of the 
///			listener and the sounds in the scene. 
///
///			This shouldn't be too hard to link into your scene-graphs by parenting
///			some form of dummy nodes underneath a transform.
///
///
///			MEGA IMPORTANT
///			==============
///
///			DirectAudio provides two ways to update the 3D sound information. The first
///			way re-evaluates the 3D sound information every time you change a setting.
///			This gets very very slow for anything more than just a simple scene. The 
///			second way basically batches up all changes you make to the sounds and
///			then re-evaluates the buffers in one go.
///
///			This gives a greatly improved speed increase, and is therefore the method 
///			that I have used within this code. What this means is that none of your
///			changes will occur until you call the function 
///
///				MediaUpdate3D()
///
///			I would assume that in a game scenario, this function would be called
///			at the start of you rendering loop. 
///
/////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////
//
//	Includes
//
/////////////////////////////////////////////////////////
#include <windows.h>
#define INITGUID
#include <dmusici.h>
#include <dsound.h>
#include <vector>
#include <cassert>
#include <iostream>

#include "MediaPlayer.h"

// force the DirectShow libs to be linked
#pragma comment(lib,"dxguid.lib")
#pragma comment(lib,"strmiids.lib")
#pragma comment(lib,"dsound.lib")

/////////////////////////////////////////////////////////
//
//	Custom Data Types
//
/////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////
/// this structure holds references to all media files loaded
///
typedef struct _SoundStruct
{
	/// ctor
	_SoundStruct() 
		: m_pSegment(NULL),
		  m_p3DAudioPath(NULL),
		  m_pDirectSoundBuffer(NULL),
		  m_pDirectSoundBuffer3D(NULL)
	{
	}

	/// the media segment that this struct represents
	IDirectMusicSegment8*	 m_pSegment;

	/// the audio path with the setting used to render this sound
	IDirectMusicAudioPath8*  m_p3DAudioPath;

	/// a direct sound buffer interface
	IDirectSoundBuffer8*	 m_pDirectSoundBuffer;

	/// a direct sound buffer interface
	IDirectSound3DBuffer8*	 m_pDirectSoundBuffer3D;

	/// the unique id of this media file
	unsigned int			 m_id;
} 
SoundStruct;



/////////////////////////////////////////////////////////
//
//	Globals
//
/////////////////////////////////////////////////////////

/// an array of all sounds that are currently loaded
std::vector< SoundStruct* > g_aSounds;

/// a direct show media file loader
IDirectMusicLoader8*		g_pLoader		= NULL;

/// a media performance controller
IDirectMusicPerformance8*	g_pPerformance  = NULL;

///	the interface for 3D listener position
IDirectSound3DListener8*	g_pListener		= NULL;
	
/////////////////////////////////////////////////////////
//
//	Internal Functions
//
/////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////
/// \brief	This function generates a new unique id that will
///			be used to differentiate between different sound files
///
unsigned int GenID() 
{
	// the id
	unsigned int id=0;

label:

	// try next var id
	++id;

	// iterate through the array of soundfiles
	std::vector< SoundStruct* >::iterator it = g_aSounds.begin();

	for( ; it != g_aSounds.end(); ++it )
	{
		/// if found, go back and try another number
		if((*it)->m_id == id) goto label;
	}

	/// return this id for it has not been used before...
	return id;
}

/////////////////////////////////////////////////////////
///	\brief	This function is a quick accessor function to retrieve a specific
///			sound file from it's node id.
///
SoundStruct* GetSound(unsigned int id)
{
	// iterate through all sound files
	std::vector< SoundStruct* >::iterator it = g_aSounds.begin();

	for( ; it != g_aSounds.end(); ++it )
	{
		// if id found, return pointer to sound
		if((*it)->m_id == id)
		{
			return *it;
		}
	}
	return NULL;
}


/////////////////////////////////////////////////////////
///	\brief	This internal function creates a new sound file, 
///			generates a unique id for it, and then adds it
///			into the global array of sound files.
///
SoundStruct* AddSound()
{
	// create a new sound file struct
	SoundStruct* pSound = new SoundStruct;

	// check the memory allocation
	assert(pSound);

	// generate a unique id for the sound
	pSound->m_id = GenID();

	// add the sound to the back of the global sound array
	g_aSounds.push_back(pSound);

	// return a pointer to the sound structure
    return pSound;
}



/////////////////////////////////////////////////////////
/// \brief	This function allows you to delete a loaded sound file
/// \param	id	-	the id of the file to delete
///
void MediaDeleteSound(unsigned int id)
{
	//	find the sound to delete by looping through the global sound array
	std::vector< SoundStruct* >::iterator it = g_aSounds.begin();
	for( ; it != g_aSounds.end(); ++it )
	{
		//	if sound found ...
		if((*it)->m_id == id)
		{
			// release the direct sound buffer interface
			if( (*it)->m_pDirectSoundBuffer )
			{
				(*it)->m_pDirectSoundBuffer->Release();
				(*it)->m_pDirectSoundBuffer = NULL;
			}

			// release the direct sound buffer 3D interface
			if( (*it)->m_pDirectSoundBuffer3D )
			{
				(*it)->m_pDirectSoundBuffer3D->Release();
				(*it)->m_pDirectSoundBuffer3D = NULL;
			}

			// release the audio path interface
			if( (*it)->m_p3DAudioPath )
			{
				(*it)->m_p3DAudioPath->Release();
				(*it)->m_p3DAudioPath = NULL;
			}
			
			// release the media segment interface
			if( (*it)->m_pSegment )
			{
				(*it)->m_pSegment->Release();
				(*it)->m_pSegment = NULL;
			}

			// delete the sound file struct
			delete *it;

			// erase the pointer from the global array of loaded sounds
			g_aSounds.erase(it);

			// we're done...
			return;
		}
	}
}

/////////////////////////////////////////////////////////
/// \brief	This function allows you to delete a loaded sound file
/// \param	id	-	the id of the file to delete
///
void MediaDeleteAllSounds(void)
{
	// iterate through all of the sounds in the sound array
	std::vector< SoundStruct* >::iterator it = g_aSounds.begin();

	for( ; it != g_aSounds.end(); ++it )
	{
		// release the direct sound buffer 3D interface
		if( (*it)->m_pDirectSoundBuffer3D )
		{
			(*it)->m_pDirectSoundBuffer3D->Release();
			(*it)->m_pDirectSoundBuffer3D = NULL;
		}

		// release the direct sound buffer interface
		if( (*it)->m_pDirectSoundBuffer )
		{
			(*it)->m_pDirectSoundBuffer->Release();
			(*it)->m_pDirectSoundBuffer = NULL;
		}

		// release the audio path interface
		if( (*it)->m_p3DAudioPath )
		{
			(*it)->m_p3DAudioPath->Release();
			(*it)->m_p3DAudioPath = NULL;
		}

		// release the media segment interface
		if( (*it)->m_pSegment )
		{
			(*it)->m_pSegment->Release();
			(*it)->m_pSegment = NULL;
		}

		// delete the sound file struct
		delete *it;

		// set pointer to NULL just in case
		*it = NULL;
	}

	// zero the size of the array
	g_aSounds.resize(0);
}



/////////////////////////////////////////////////////////
/// \brief	initialises DirectAudio
///
void MediaInit(void) 
{
	CoInitialize(NULL);

	// create a DirectMusicLoader interface
	CoCreateInstance(CLSID_DirectMusicLoader, NULL, 
		CLSCTX_INPROC, IID_IDirectMusicLoader8,
		(void**)&g_pLoader);

	// create a DirectMusicPerformance interface
	CoCreateInstance(CLSID_DirectMusicPerformance, NULL,
		CLSCTX_INPROC, IID_IDirectMusicPerformance8,
		(void**)&g_pPerformance );

    // Initialize the performance with the standard audio path.
    // This initializes both DirectMusic and DirectSound and 
    // sets up the synthesizer. 
    g_pPerformance->InitAudio( NULL, NULL, NULL, 
                               DMUS_APATH_DYNAMIC_STEREO, 64,
                               DMUS_AUDIOF_ALL, NULL );


}


/////////////////////////////////////////////////////////
/// \brief	This function is used to cleanup DirectShow 
///			and release any interfaces we have gathered
///
void MediaCleanup(void)
{

	// release the 3D listener interface
	if( g_pListener )
	{

		// release the COM object
		g_pListener->Release();

		// be safe rather than sorry
		g_pListener = NULL;
	}
	
	// release the media performance interface
	if( g_pPerformance )
	{
		// stops playback of all files
		g_pPerformance->Stop(
							NULL,	// Stop all segments.
							NULL,	// Stop all segment states.
							0,		// Do it immediately.
							0		// Flags.
							);


		// make sure that all sounds have been deleted
		MediaDeleteAllSounds();

		// close down DirectAudio
		g_pPerformance->CloseDown();

		// release the COM object
		g_pPerformance->Release();

		// be safe rather than sorry
		g_pPerformance = NULL;
	}

	// release the playback controller
	if( g_pLoader )
	{

		// release the COM object
		g_pLoader->Release();

		// be safe rather than sorry
		g_pLoader = NULL;
	}
	
	// Unload DirectAudio
	CoUninitialize();
}




/////////////////////////////////////////////////////////
/// \brief	call this to pause playback of the mp3
/// \param	szFile	-	the name of the file to load
///
unsigned int MediaLoad(char filename[])
{
	// DirectAudio requires unicode (16bit) characters rather than 8bit ascii
	//
	WCHAR wFile[MAX_PATH];

	// convert ascii to unicode
	//
	MultiByteToWideChar(CP_ACP, 0, filename, -1, wFile, MAX_PATH);

	// create a new sound file
	//
	SoundStruct* pSound = AddSound();

	// if you wish to set parameters for sounds individually, you have to 
	// create seperate audio paths for each item.
	//
	g_pPerformance->CreateStandardAudioPath
		( 
			DMUS_APATH_DYNAMIC_3D,  // Path type.
			64,                     // Number of performance channels.
			TRUE,                   // Activate now.
			&pSound->m_p3DAudioPath // Pointer that receives audiopath.
		);

	// we want to get an interface to the directsound buffer that will be used for 
	// our wave files. This interface will allow us to query and set the volume
	// and panning for the sound.
	//
	pSound->m_p3DAudioPath->GetObjectInPath
		( 
			0,									  // Performance channel.
			DMUS_PATH_BUFFER,					  // Stage in the path.
			0,									  // Index of buffer in chain.
			GUID_NULL,							  // Class of object.
			0,									  // Index of object in buffer; ignored.
			IID_IDirectSoundBuffer,				  // GUID of desired interface.
			(void**)&pSound->m_pDirectSoundBuffer // Pointer that receives interface.
		);

	if(!pSound->m_pDirectSoundBuffer)
	{
		std::cout << "unable to get sound buffer iface " << std::endl;
	}


	// we want to get an interface to the directsound buffer that will be used for 
	// our wave files. This interface will allow us to query and set the volume
	// and panning for the sound.
	//
	pSound->m_p3DAudioPath->GetObjectInPath
		( 
			0,									  // Performance channel.
			DMUS_PATH_BUFFER,					  // Stage in the path.
			0,									  // Index of buffer in chain.
			GUID_NULL,							  // Class of object.
			0,									  // Index of object in buffer; ignored.
			IID_IDirectSound3DBuffer,				  // GUID of desired interface.
			(void**)&pSound->m_pDirectSoundBuffer3D // Pointer that receives interface.
		);

	if(!pSound->m_pDirectSoundBuffer3D)
	{
		std::cout << "unable to get 3D sound buffer iface " << std::endl;
	}


	// we want to get an interface to the directsound buffer that will be used for 
	// our wave files. This interface will allow us to query and set the volume
	// and panning for the sound.
	//
	pSound->m_p3DAudioPath->GetObjectInPath
		( 
			0,					  // Performance channel.
			DMUS_PATH_PRIMARY_BUFFER,					  // Stage in the path.
			0,									  // Index of buffer in chain.
			GUID_NULL,							  // Class of object.
			0,									  // Index of object in buffer; ignored.
			IID_IDirectSound3DListener,			  // GUID of desired interface.
			(void**)&g_pListener					// Pointer that receives interface.
		);

	if(!g_pListener)
	{
		std::cout << "unable to get 3D sound listener iface " << std::endl;
	}

	// load the specified file into the segment provided
	//
	if (FAILED(g_pLoader->LoadObjectFromFile(
		CLSID_DirectMusicSegment,	 // Class identifier.
		IID_IDirectMusicSegment8,	 // ID of desired interface.
		wFile,						 // Filename.
		(void**) &pSound->m_pSegment // Pointer that receives interface.
		)))
	{
		printf("Could not locate requested media %s\n",filename);
		return 0;
	}

    // Download the segment's instruments to the synthesizer (if midi file)
	//
    pSound->m_pSegment->Download( g_pPerformance );

	// return the sound id
	//
	return pSound->m_id;
}


/////////////////////////////////////////////////////////
/// \brief	call this to start playback of the mp3
///	\param	id	-	the id of the sound to play
///
void MediaPlay(unsigned int id)
{
	// retrieve the sound file structure
	SoundStruct* pSound = GetSound(id);

	// if found
	if (pSound)
	{
		// start playback of the file
		g_pPerformance->PlaySegmentEx(
								pSound->m_pSegment,	 // Segment to play.
								NULL,				 // Used for songs; not implemented.
								NULL,				 // For transitions. 
								0,					 // Flags.
								0,					 // Start time; 0 is immediate.
								NULL,				 // Pointer that receives segment state.
								NULL,				 // Object to stop.
								pSound->m_p3DAudioPath				 // Audiopath, if not default.

							);


	}
}

/////////////////////////////////////////////////////////
/// \brief	call this to start playback of the mp3
///	\param	id	-	the id of the sound to play
///
void MediaStop(unsigned int id)
{
	// retrieve the sound file structure
	SoundStruct* pSound = GetSound(id);

	// if found
	if (pSound) 
	{
		// stop playback of the file
		g_pPerformance->StopEx(
							  pSound->m_pSegment,// Segment to play.
							  0,				 // Used for songs; not implemented.
							  0);  
	}
}





/////////////////////////////////////////////////////////
/// \brief	call this to set the current volume level. Valid values are 
///			between -10000 and 0.
///	\param	id	-	the id of the sound to set the volume for
///	\param	vol	-	the new volume level (-10000 to 0)
///
void MediaSetVolume(unsigned int id,long vol)
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		// if the interface to the direct sound buffer has been found
		if( pSound->m_pDirectSoundBuffer )
		{
			pSound->m_pDirectSoundBuffer->SetVolume(vol);
		}
	}
}

/////////////////////////////////////////////////////////
/// \brief	call this to get the current volume level. Valid values are 
///			between -10000 and 0.
///	\param	id	-	the id of the sound to query
///	\return	the volume
///
long MediaGetVolume(unsigned int id)
{
	// will hold the volume
	long vol = 0;

	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		// if the interface to the direct sound buffer has been found
		if( pSound->m_pDirectSoundBuffer )
		{
			// set the sounds balance
			pSound->m_pDirectSoundBuffer->GetVolume(&vol);
		}
	}

	// return the volume
	return vol;
}


/////////////////////////////////////////////////////////
/// \brief	call this to set the current balance. Valid values are 
///			between -10000 and 10000.
///	\param	id	-	the id of the sound to set the balance for
///	\param	bal	-	the new balance level (-10000 to 10000)
///
void MediaSetBalance(unsigned int id,long bal)
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		// if the interface to the direct sound buffer has been found
		if( pSound->m_pDirectSoundBuffer )
		{
			// set the sounds balance
			pSound->m_pDirectSoundBuffer->SetPan(bal);
		}
	}
}


/////////////////////////////////////////////////////////
/// \brief	call this to get the current balance. Valid values are 
///			between -10000 and 10000.
///	\param	id	-	the id of the sound to query
///	\return	the balance
///
long MediaGetBalance(unsigned int id)
{
	// will hold the balance
	long bal = 0;

	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		// if the interface to the direct sound buffer has been found
		if( pSound->m_pDirectSoundBuffer )
		{
			// set the sounds balance
			pSound->m_pDirectSoundBuffer->GetPan(&bal);
		}
	}

	// return the balance
	return bal;
}



/////////////////////////////////////////////////////////
/// \brief	call this to set the current frequency. Valid values are 
///			between 100 and 100000.
///	\param	id	-	the id of the sound to set the frequency for
///	\param	frq	-	the new frequency level (100 to 100000)
///
void MediaSetFrequency(unsigned int id,long frq)
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		// if the interface to the direct sound buffer has been found
		if( pSound->m_pDirectSoundBuffer )
		{
			// set the sounds frequency
			pSound->m_pDirectSoundBuffer->SetFrequency(frq);
		}
	}
}


/////////////////////////////////////////////////////////
/// \brief	call this to get the current frequency. Valid values are 
///			between 100 and 100000.
///	\param	id	-	the id of the sound to query
///	\return	the frequency
///
long MediaGetFrequency(unsigned int id)
{
	// will hold the frequency
	DWORD frq = 0;

	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		// if the interface to the direct sound buffer has been found
		if( pSound->m_pDirectSoundBuffer )
		{
			// get the sounds frequency
			pSound->m_pDirectSoundBuffer->GetFrequency(&frq);
		}
	}

	// return the frequency
	return frq;
}



/////////////////////////////////////////////////////////////////////////
//
//	3D Sound Controls [Positional Sound Controls]
//
/////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////
/// \brief	The inside and ouside cone value are specified in angles. It
///			allows you to specify a drop off of the sound between certain 
///			ranges. Th imside cone angle specifies the angle of the cone
///			inside which the sound level is normal. The ouside cone angle
///			specifies the angle at which we reach the outside volume. Between
///			these two angles the volume of the sound will vary.
/// \param	id	-	the id of the sound to use
/// \param	pInsideConeAngle	-	the inside cone angle (in degrees)
///	\param	pOutsideConeAngle	-	the outside cone angle (in degrees)
///
void MediaGetConeAngles( unsigned int id, long* pInsideConeAngle, long* pOutsideConeAngle )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			DWORD inside;
			DWORD outside;
			pSound->m_pDirectSoundBuffer3D->GetConeAngles(&inside,&outside);

			if(pInsideConeAngle) {
				*pInsideConeAngle = inside;
			}

			if(pOutsideConeAngle) {
				*pOutsideConeAngle = outside;
			}			
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/// \brief	This function gets the orientation vector of the sound cone
/// \param	id	-	the id of the sound to use
/// \param	orient	-	this will be filled with the 3D direction vector
///
void MediaGetConeOrientation( unsigned int id, float orient[] )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			D3DVECTOR vec;
			pSound->m_pDirectSoundBuffer3D->GetConeOrientation(&vec);
			orient[0] = vec.x;
			orient[1] = vec.y;
			orient[2] = vec.z;	
		}
	}
}


/////////////////////////////////////////////////////////////////////////
/// \brief	retrieves the volume level at the outside angle of the cone
/// \param	id	-	the id of the sound to use
///	\return	The volume level at the outside angle of the cone
///
long MediaGetConeOutsideVolume( unsigned int id )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			long val;
			pSound->m_pDirectSoundBuffer3D->GetConeOutsideVolume(&val);
			return val;
		}
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////
/// \brief	This function retrieves the position in 3D space, of the sound location
/// \param	id	-	the id of the sound to use
/// \param	position	-	this will be filled with a 3D coordinate of the sound position
///
void MediaGetSoundPosition( unsigned int id, float position[] )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			D3DVECTOR vec;
			pSound->m_pDirectSoundBuffer3D->GetPosition(&vec);
			position[0] = vec.x;
			position[1] = vec.y;
			position[2] = vec.z;	
		}
	}
}


/////////////////////////////////////////////////////////////////////////
/// \brief	This gets the speed vector of the sound (for the doppler effect)
/// \param	id	-	the id of the sound to use
/// \param	velocity	-	this will be filled with a 3D vector 
///							representing the speed of the sound.
///
void MediaGetSoundVelocity( unsigned int id, float velocity[] )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			D3DVECTOR vec;
			pSound->m_pDirectSoundBuffer3D->GetVelocity(&vec);
			velocity[0] = vec.x;
			velocity[1] = vec.y;
			velocity[2] = vec.z;	
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/// \brief	Gets the maximum distance (in meters) at which the sound can be heard
/// \param	id	-	the id of the sound to use
///	\return	the maximum distance at which the sound can be heard 
///
float MediaGetMaxDistance( unsigned int id )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			float ret = 0.0f;
			pSound->m_pDirectSoundBuffer3D->GetMaxDistance(&ret);
			return ret;			
		}
	}
	return 0.0f;
}


/////////////////////////////////////////////////////////////////////////
/// \brief	Gets the minimum distance (in meters) of this sound
/// \param	id	-	the id of the sound to use
///	\return	the minimum distance of the sound 
///
float MediaGetMinDistance( unsigned int id )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			float ret = 0.0f;
			pSound->m_pDirectSoundBuffer3D->GetMinDistance(&ret);
			return ret;		
		}
	}
	return 0.0f;
}



/////////////////////////////////////////////////////////////////////////
/// \brief	The inside and ouside cone value are specified in angles. It
///			allows you to specify a drop off of the sound between certain 
///			ranges. Th imside cone angle specifies the angle of the cone
///			inside which the sound level is normal. The ouside cone angle
///			specifies the angle at which we reach the outside volume. Between
///			these two angles the volume of the sound will vary.
/// \param	id					-	the id of the sound to use
/// \param	InsideConeAngle		-	the inside cone angle (in degrees)
///	\param	OutsideConeAngle	-	the outside cone angle (in degrees)
///
void MediaSetConeAngles( unsigned int id, long InsideConeAngle, long OutsideConeAngle )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			pSound->m_pDirectSoundBuffer3D->SetConeAngles(InsideConeAngle,OutsideConeAngle,DS3D_DEFERRED);
		}
	}
}


/////////////////////////////////////////////////////////////////////////
/// \brief	This function sets the orientation vector of the sound cone
/// \param	id		-	the id of the sound to use
/// \param	orient	-	the new 3D cone orientation vector
///
void MediaSetConeOrientation( unsigned int id, float orient[] )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			pSound->m_pDirectSoundBuffer3D->SetConeOrientation(orient[0],orient[1],orient[2],DS3D_DEFERRED);
		}
	}
}


/////////////////////////////////////////////////////////////////////////
/// \brief	sets the volume level at the outside angle of the cone
/// \param	id		-	the id of the sound to use
///	\param	volume	-	The volume level at the outside angle of the cone
///
void MediaSetConeOutsideVolume( unsigned int id, long volume )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			pSound->m_pDirectSoundBuffer3D->SetConeOutsideVolume(volume,DS3D_DEFERRED);
		}
	}
}


/////////////////////////////////////////////////////////////////////////
/// \brief	This function sets the position in 3D space, of the sound 
/// \param	id			-	the id of the sound to use
/// \param	position	-	the new location of the sound
///
void MediaSetSoundPosition( unsigned int id, float position[] )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			pSound->m_pDirectSoundBuffer3D->SetPosition(position[0],position[1],position[2],DS3D_DEFERRED);
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/// \brief	This sets the speed vector of the sound (for the doppler effect)
/// \param	id			-	the id of the sound to use
/// \param	velocity	-	the speed of the sound as a 3D vector.
///
void MediaSetSoundVelocity( unsigned int id, float velocity[] )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			pSound->m_pDirectSoundBuffer3D->SetVelocity(velocity[0],velocity[1],velocity[2],DS3D_DEFERRED);
		}
	}
}


/////////////////////////////////////////////////////////////////////////
/// \brief	Sets the maximum distance (in meters) at which the sound can be heard
/// \param	id		-	the id of the sound to use
///	\param	dist	-	the maximum distance at which the sound can be heard 
///						(use DISTANCE_INFINITE for an infinate sound)
///
void MediaSetMaxDistance( unsigned int id, float dist )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			pSound->m_pDirectSoundBuffer3D->SetMaxDistance(dist,DS3D_DEFERRED);
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/// \brief	Sets the minimum distance (in meters) of this sound
/// \param	id		-	the id of the sound to use
///	\param	dist	-	the minimum distance of the sound 
///
void MediaSetMinDistance( unsigned int id, float dist )
{
	// get pointer to the sound structure
	SoundStruct* pSound = GetSound(id);

	// if sound id is valid and sound was found
	if (pSound) 
	{
		//	make sure we have a 3D interface
		if (pSound->m_pDirectSoundBuffer3D) 
		{
			pSound->m_pDirectSoundBuffer3D->SetMinDistance(dist,DS3D_DEFERRED);
		}
	}
}



/////////////////////////////////////////////////////////////////////////
///	\brief	gets the doppler factor for the specified sound
/// \return	the doppler factor
///
float MediaGetDopplerFactor( void )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		float f;
		g_pListener->GetDopplerFactor(&f);
		return f;
	}
	
	return 0.0f;
}

/////////////////////////////////////////////////////////////////////////
///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	frontVec	-	to be filled with the direction in which the listener is facing
///							this should be an array of 3 floats
/// \param	upVec		-	to be filled with the up direction of the listener 
///							this should be an array of 3 floats
///
void  MediaGetOrientation( float frontVec[], float upVec[] )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		D3DVECTOR vec1;
		D3DVECTOR vec2;
		g_pListener->GetOrientation(&vec1,&vec2);

		frontVec[0] = vec1.x;
		frontVec[1] = vec1.y;
		frontVec[2] = vec1.z;

		upVec[0] = vec2.x;
		upVec[1] = vec2.y;
		upVec[2] = vec2.z;
	}
}

/////////////////////////////////////////////////////////////////////////
///	\brief	This function gets the position of the listener
/// \param	position	-	to be filled with the position of the listener
///							this should be an array of 3 floats
///
void  MediaGetPosition( float position[] )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		D3DVECTOR vec;
		g_pListener->GetPosition(&vec);

		position[0] = vec.x;
		position[1] = vec.y;
		position[2] = vec.z;
	}
}

/////////////////////////////////////////////////////////////////////////
///	\brief	This function returns the roll off factor for the listener
/// \return the roll off factor
///
float MediaGetRollOffFactor( void )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		float f;
		g_pListener->GetRolloffFactor(&f);
		return f;
	}
	
	return 0.0f;
}

/////////////////////////////////////////////////////////////////////////
///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	velocity	-	to be filled with the velocity in which the listener is moving
///							this should be an array of 3 floats
///
void  MediaGetVelocity( float velocity[] )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		D3DVECTOR vec;
		g_pListener->GetVelocity(&vec);

		velocity[0] = vec.x;
		velocity[1] = vec.y;
		velocity[2] = vec.z;
	}
}

/////////////////////////////////////////////////////////////////////////
///	\brief	sets the doppler factor for the specified sound
///	\param	val	-	the new doppler factor value
///
void MediaSetDopplerFactor( float val )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		g_pListener->SetDopplerFactor(val,DS3D_DEFERRED);
	}
}


/////////////////////////////////////////////////////////////////////////
///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	frontVec	-	to be filled with the direction in which the listener is facing
///							this should be an array of 3 floats
/// \param	upVec		-	to be filled with the up direction of the listener 
///							this should be an array of 3 floats
///
void  MediaSetOrientation( float frontVec[], float upVec[] )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		g_pListener->SetOrientation(frontVec[0],frontVec[1],frontVec[2],upVec[0],upVec[1],upVec[2],DS3D_DEFERRED);
	}
}

/////////////////////////////////////////////////////////////////////////
///	\brief	This function gets the position of the listener
/// \param	position	-	to be filled with the position of the listener
///							this should be an array of 3 floats
///
void  MediaSetPosition( float position[] )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		g_pListener->SetPosition(position[0],position[1],position[2],DS3D_DEFERRED);
	}
}

/////////////////////////////////////////////////////////////////////////
///	\brief	This function returns the roll off factor for the listener
///	\param	val	-	the new roll off factor value
///
void MediaSetRollOffFactor( float val )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		g_pListener->SetRolloffFactor(val,DS3D_DEFERRED);
	}
}

/////////////////////////////////////////////////////////////////////////
///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	velocity	-	to be filled with the velocity in which the listener is moving
///							this should be an array of 3 floats
///
void  MediaSetVelocity( float velocity[] )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		g_pListener->SetVelocity(velocity[0],velocity[1],velocity[2],DS3D_DEFERRED);
	}
}

/////////////////////////////////////////////////////////////////////////
/// \brief	because evaluation of 3D buffers takes a bit of time, rather than
///			do it every time one of the set functions is called, i've set
///			it up so that all changes are deffered until this function is called.
///			This allows you to make lots of changes to the 3D settings and
///			then perform only a single re-calculation. If you pass zero into
///			this function then it will update all of the sound files.
///
///			You must call this after changing any 3D settings for a sound or
///			the listener.
///
void MediaUpdate3D( void )
{
	//	make sure we have a 3D interface
	if (g_pListener) 
	{
		g_pListener->CommitDeferredSettings();
	}
}



