/////////////////////////////////////////////////////////////////////////
///
///	\file	MediaPlayer.h
///	\author	Rob Bateman
///	\date	09-03-2003
///	\brief	This file defines a set of functions that hide away DirectAudio
///			behind a nice C interface. These set of functions allow you
///			to load a number of media files (wav, midi etc) for sound
///			effects and background music (midi files only though!).
///
///	\note	
///			I assume that this supports all native media types in windows.
///			(ie, divx will work if the codecs are installed on the system)
///
///
///			Usage
///			-----
///
///			You must ensure that you call MediaInit() before you call any
///			other media function. 
/// 
///			When you have finished with DirectAudio, call MediaCleanup().
/// 
///			In order to load and play a media file, you should first call
///			MediaLoad() specifying the file you wish to load. This function
///			will return to you an ID number with which you can refer to 
///			the sound. For example
///
///
///				// load the sound "SomeSound.wav" and store it's id num in "SoundID"
///				unsigned int SoundID = MediaLoad("SomeSound.wav");
///
/// 
///			you can then start and stop playback using MediaPlay() and 
///			MediaStop(). ie, 
/// 
///
///				// this starts playback of the wave file
///				MediaPlay( SoundID );
///
///			
///			You can also make use of the following sets of functions to 
///			change properties of the sound playback :
///
///
///			File Management Controls
///			------------------------
///
///			unsigned int MediaLoad ( char filename[] )	
///			void		 MediaDelete ( unsigned int id );			
///			void		 MediaDeleteAll ( );
///	
///
///			Volume Controls		-	valid values -10000 -> 0
///			---------------
///
///			void		MediaSetVolume ( unsigned int id, long balance )		sets the current volume level for the sound
///			long		MediaGetVolume ( unsigned int id )						gets current volume level of the sound
///
///
///			Balance Controls	-	valid values -10000 -> 10000
///			----------------
///
///			void		MediaSetBalance ( unsigned int id, long balance )		sets the current balance amount for the sound
///			long		MediaGetBalance ( unsigned int id )						gets current balance amount of the sound
///
///
///			Frequency Controls	-	valid values 100 -> 100000
///			------------------
///
///			void		MediaSetFrequency ( unsigned int id, long frequency )	sets the current pitch of the sound
///			long		MediaGetFrequency ( unsigned int id )					gets the current pitch of the sound
///
///
///
///			3D Sound Controls [Sound Position Controls]
///			-------------------------------------------
///
///			void		MediaGetConeAngles ( unsigned int id, long* pInsideConeAngle, long* pOutsideConeAngle );
///
///			void		MediaGetConeOrientation ( unsigned int id, float orient[] );
///
///			long		MediaGetConeOutsideVolume ( unsigned int id );
///
///			void		MediaGetPosition ( unsigned int id, float position[] );
///
///			void		MediaGetVelocity ( unsigned int id, float velocity[] );
///
///			float		MediaGetMaxDistance ( unsigned int id );
///		
///			float		MediaGetMinDistance ( unsigned int id );
///
///			
///			void		MediaSetConeAngles ( unsigned int id, long InsideConeAngle, long OutsideConeAngle );
///
///			void		MediaSetConeOrientation ( unsigned int id, float orient[] );
///
///			void		MediaSetConeOutsideVolume ( unsigned int id, long volume );
///
///			void		MediaSetPosition ( unsigned int id, float position[] );
///
///			void		MediaSetVelocity ( unsigned int id, float velocity[] );
///
///			void		MediaSetMaxDistance ( unsigned int id, float dist );
///
///			void		MediaSetMinDistance ( unsigned int id, float dist );
///			
///
///
///			3D Sound Controls [Listener Position Controls]
///			----------------------------------------------
///
///			float		MediaGetDopplerFactor ( );
///
///			void		MediaGetOrientation ( float frontVec[], float upVec[] );
///
///			void		MediaGetPosition ( float position[] );
///
///			float		MediaGetRollOffFactor ( );
///
///			void		MediaGetVelocity ( float velocity[] );
///
///
///			void		MediaSetDopplerFactor ( float val );
///
///			void		MediaSetOrientation ( float frontVec[], float upVec[] );
///
///			void		MediaSetPosition ( float position[] );
///
///			void		MediaSetRollOffFactor ( float val );
///
///			void		MediaSetVelocity (  float velocity[] );
///
///
///
///			3D Sound Update Func
///			--------------------
///
///			void		MediaUpdate3D ();
///
///
///
///			3D Sound Controls	[ READ ALL OF THIS BEFORE USE ]
///			====================================================
///
///
///			Essentually the 3D sound controls are split into 2 catagories : 
///
///			1. Positioning The Listener (Ear if you will) in 3D space
///			2. Positioning The Sound in 3D space
///
///			There are three main effects available as part of 3D sound are
///
///			1. Position - a sound may be placed far away or nearby. The volume of 
///					that sound will depend on this distance between the listener
///					and the sound source location. 
///
///			2. Orientation - a sound may occur behind or infront of you 
///					depending on the direction of the listener,
///
///			3. Doppler Effect - the doppler effect is the change of a sounds
///					frequency which is caused due to the sound source or listener
///					travelling at speed. (ie, the change in pitch of a police siren
///					as it speeds past you ).
///
///			
///			All of these effects are calculated automatically and only require you
///			to specify things like the position, velocity and orientation of the 
///			listener and the sounds in the scene. 
///
///			This shouldn't be too hard to link into your scene-graphs by parenting
///			some form of dummy nodes underneath a transform.
///
///
///			MEGA IMPORTANT
///			==============
///
///			DirectAudio provides two ways to update the 3D sound information. The first
///			way re-evaluates the 3D sound information every time you change a setting.
///			This gets very very slow for anything more than just a simple scene. The 
///			second way basically batches up all changes you make to the sounds and
///			then re-evaluates the buffers in one go.
///
///			This gives a greatly improved speed increase, and is therefore the method 
///			that I have used within this code. What this means is that none of your
///			changes will occur until you call the function 
///
///				MediaUpdate3D()
///
///			I would assume that in a game scenario, this function would be called
///			at the start of you rendering loop. 
///
/////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////
//
//	Sentry
//
/////////////////////////////////////////////////////////////////////////


#pragma once

#ifndef _MEDIA_PLAYER_H_
#define _MEDIA_PLAYER_H_


/////////////////////////////////////////////////////////////////////////
//
//	Consts
//
/////////////////////////////////////////////////////////////////////////

///	an infinite distnce for sounds ....
///
#define DISTANCE_INFINITE 1000000000.0f




/////////////////////////////////////////////////////////////////////////
//
//	Direct Audio Initialisation/UnInitialisation
//
/////////////////////////////////////////////////////////////////////////

/// \brief	initialises DirectAudio
///
extern void MediaInit(void);


/// \brief	This function is used to cleanup DirectShow 
///			and release any interfaces we have gathered
///
extern void MediaCleanup(void);





/////////////////////////////////////////////////////////////////////////
//
//	File Management Functions
//
/////////////////////////////////////////////////////////////////////////

/// \brief	call this to load the specified media file. This function will
///			return the id of the sound which you require in order to pass
///			into the functions that control playback.
/// \param	filename	-	the name of the file to load
/// \return	a unique id that represents the sound that was loaded.
///
extern unsigned int MediaLoad(char filename[]);


/// \brief	This function allows you to delete a loaded sound file
/// \param	id	-	the id of the file to delete
///
extern void MediaDelete(unsigned int id);

/// \brief	This function allows you to delete a loaded sound file
/// \param	id	-	the id of the file to delete
///
extern void MediaDeleteAll(void);





/////////////////////////////////////////////////////////////////////////
//
//	Playback Control
//
/////////////////////////////////////////////////////////////////////////

/// \brief	call this to start playback of the mp3
///	\param	id	-	the id of the sound to play
///
extern void MediaPlay(unsigned int);


/// \brief	call this to start playback of the mp3
///	\param	id	-	the id of the sound to play
///
extern void MediaStop(unsigned int);





/////////////////////////////////////////////////////////////////////////
//
//	Sound Settings
//
/////////////////////////////////////////////////////////////////////////

/// \brief	call this to set the current volume level. Valid values are 
///			between -10000 and 0.
///	\param	id	-	the id of the sound to set the volume for
///	\param	vol	-	the new volume level (-10000 to 0)
///
extern void MediaSetVolume(unsigned int,long);


/// \brief	call this to get the current volume level. Valid values are 
///			between -10000 and 0.
///	\param	id	-	the id of the sound to query
///	\return	the volume
///
extern long MediaGetVolume(unsigned int);


/// \brief	call this to set the current balance. Valid values are 
///			between -10000 and 10000.
///	\param	id	-	the id of the sound to set the balance for
///	\param	bal	-	the new balance level (-10000 to 10000)
///
extern void MediaSetBalance(unsigned int,long);


/// \brief	call this to get the current balance. Valid values are 
///			between -10000 and 10000.
///	\param	id	-	the id of the sound to query
///	\return	the balance
///
extern long MediaGetBalance(unsigned int);


/// \brief	call this to set the current frequency. Valid values are 
///			between 100 and 100000.
///	\param	id	-	the id of the sound to set the frequency for
///	\param	frq	-	the new frequency level (100 to 100000)
///
extern void MediaSetFrequency(unsigned int,long);


/// \brief	call this to get the current frequency. Valid values are 
///			between 100 and 100000.
///	\param	id	-	the id of the sound to query
///	\return	the frequency
///
extern long MediaGetFrequency(unsigned int);






/////////////////////////////////////////////////////////////////////////
///
///
///	3D Sound Controls	[ READ ALL OF THIS BEFORE USE ]
///	====================================================
///
///
///	Essentually the 3D sound controls are split into 2 catagories : 
///
///	1. Positioning The Listener (Ear if you will) in 3D space
///	2. Positioning The Sound in 3D space
///
///	There are four main effects available as part of Direct Sound 3D :
///
///	1. Position - a sound may be placed far away or nearby. The volume of 
///			that sound will depend on this distance between the listener
///			and the sound source location. 
///
///	2. Orientation - a sound may occur behind or infront of you 
///			depending on the direction of the listener,
///
///	3. Doppler Effect - the doppler effect is the change of a sounds
///			frequency which is caused due to the sound source or listener
///			travelling at speed. (ie, the change in pitch of a police siren
///			as it speeds past you ).
/// 
/// 4, Sound Cones - A sound can be travelling in a cone, ie a speaker
///			projects sound out from it in a cone shape, and the sound
///			is therefore strongest when you are standing infront of the 
///			cone.
///
///			
///	All of these effects are calculated automatically and only require you
///	to specify things like the position, velocity and orientation of the 
///	listener and the sounds in the scene. 
///
///	This shouldn't be too hard to link into your scene-graphs by parenting
///	some form of dummy nodes underneath a transform. It may be worth using 
///	a naming convention in maya to use locator nodes to place sounds within
/// a 3D scene.
///
///
///	MEGA IMPORTANT
///	==============
///
///	DirectAudio provides two ways to update the 3D sound information. The first
///	way re-evaluates the 3D sound information every time you change a setting.
///	This gets very very slow for anything more than just a simple scene. The 
///	second way basically batches up all changes you make to the sounds and
///	then re-evaluates the buffers in one go.
///
///	This gives a greatly improved speed increase, and is therefore the method 
///	that I have used within this code. What this means is that none of your
///	changes will occur until you call the function 
///
///		MediaUpdate3D()
///
///	I would assume that in a game scenario, this function would be called
///	at the start of you rendering loop. 
///
/////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////
/// \brief	because evaluation of 3D buffers takes a bit of time, rather than
///			do it every time one of the set functions is called, i've set
///			it up so that all changes are deffered until this function is called.
///			This allows you to make lots of changes to the 3D settings and
///			then perform only a single re-calculation. If you pass zero into
///			this function then it will update all of the sound files.
///
///			You must call this after changing any 3D settings for a sound.
///
/// \param	id	-	the id of the sound to update or 0 if you want all sounds to be updated
///
extern void MediaUpdate3D( void );



/////////////////////////////////////////////////////////////////////////
//
//	3D Sound Controls [Listener Controls]
//
/////////////////////////////////////////////////////////////////////////

///	\brief	gets the doppler factor for the specified sound
/// \return	the doppler factor
///
extern float MediaGetDopplerFactor( void );

///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	frontVec	-	to be filled with the direction in which the listener is facing
///							this should be an array of 3 floats
/// \param	upVec		-	to be filled with the up direction of the listener 
///							this should be an array of 3 floats
///
extern void  MediaGetOrientation( float frontVec[], float upVec[] );

///	\brief	This function gets the position of the listener
/// \param	position	-	to be filled with the position of the listener
///							this should be an array of 3 floats
///
extern void  MediaGetPosition( float position[] );

///	\brief	This function returns the roll off factor for the listener
/// \return the roll off factor
///
extern float MediaGetRollOffFactor( void );

///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	velocity	-	to be filled with the velocity in which the listener is moving
///							this should be an array of 3 floats
///
extern void  MediaGetVelocity( float velocity[] );

///	\brief	sets the doppler factor for the specified sound
///	\param	val	-	the new doppler factor value
///
extern void MediaSetDopplerFactor( float val );

///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	frontVec	-	to be filled with the direction in which the listener is facing
///							this should be an array of 3 floats
/// \param	upVec		-	to be filled with the up direction of the listener 
///							this should be an array of 3 floats
///
extern void  MediaSetOrientation( float frontVec[], float upVec[] );

///	\brief	This function gets the position of the listener
/// \param	position	-	to be filled with the position of the listener
///							this should be an array of 3 floats
///
extern void  MediaSetPosition( float position[] );

///	\brief	This function returns the roll off factor for the listener
///	\param	val	-	the new roll off factor value
///
extern void MediaSetRollOffFactor( float val );

///	\brief	This function gets information about the direction in which the sound 
///			listener is facing. 
/// \param	velocity	-	to be filled with the velocity in which the listener is moving
///							this should be an array of 3 floats
///
extern void  MediaSetVelocity( float velocity[] );


/////////////////////////////////////////////////////////////////////////
//
//	3D Sound Controls [Positional Sound Controls]
//
/////////////////////////////////////////////////////////////////////////	

/// \brief	The inside and ouside cone value are specified in angles. It
///			allows you to specify a drop off of the sound between certain 
///			ranges. Th imside cone angle specifies the angle of the cone
///			inside which the sound level is normal. The ouside cone angle
///			specifies the angle at which we reach the outside volume. Between
///			these two angles the volume of the sound will vary.
/// \param	id					-	the id of the sound to use
/// \param	InsideConeAngle		-	the inside cone angle (in degrees)
///	\param	OutsideConeAngle	-	the outside cone angle (in degrees)
///
extern void MediaSetConeAngles( unsigned int id, long InsideConeAngle, long OutsideConeAngle );

/// \brief	This function sets the orientation vector of the sound cone
/// \param	id		-	the id of the sound to use
/// \param	orient	-	the new 3D cone orientation vector
///
extern void MediaSetConeOrientation( unsigned int id, float orient[] );

/// \brief	sets the volume level at the outside angle of the cone
/// \param	id		-	the id of the sound to use
///	\param	volume	-	The volume level at the outside angle of the cone
///
extern void MediaSetConeOutsideVolume( unsigned int id, long volume );

/// \brief	This function sets the position in 3D space, of the sound 
/// \param	id			-	the id of the sound to use
/// \param	position	-	the new location of the sound
///
extern void MediaSetSoundPosition( unsigned int id, float position[] );

/// \brief	This sets the speed vector of the sound (for the doppler effect)
/// \param	id			-	the id of the sound to use
/// \param	velocity	-	the speed of the sound as a 3D vector.
///
extern void MediaSetSoundVelocity( unsigned int id, float velocity[] );

/// \brief	Sets the maximum distance (in meters) at which the sound can be heard
/// \param	id		-	the id of the sound to use
///	\param	dist	-	the maximum distance at which the sound can be heard 
///						(use DISTANCE_INFINITE for an infinate sound)
///
extern void MediaSetMaxDistance( unsigned int id, float dist );

/// \brief	Sets the minimum distance (in meters) of this sound
/// \param	id		-	the id of the sound to use
///	\param	dist	-	the minimum distance of the sound 
///
extern void MediaSetMinDistance( unsigned int id, float dist );




/// \brief	The inside and ouside cone value are specified in angles. It
///			allows you to specify a drop off of the sound between certain 
///			ranges. Th imside cone angle specifies the angle of the cone
///			inside which the sound level is normal. The ouside cone angle
///			specifies the angle at which we reach the outside volume. Between
///			these two angles the volume of the sound will vary.
/// \param	id	-	the id of the sound to use
/// \param	pInsideConeAngle	-	the inside cone angle (in degrees)
///	\param	pOutsideConeAngle	-	the outside cone angle (in degrees)
///
extern void MediaGetConeAngles( unsigned int id, long* pInsideConeAngle, long* pOutsideConeAngle );

/// \brief	This function gets the orientation vector of the sound cone
/// \param	id	-	the id of the sound to use
/// \param	orient	-	this will be filled with the 3D direction vector
///
extern void MediaGetConeOrientation( unsigned int id, float orient[] );

/// \brief	retrieves the volume level at the outside angle of the cone
/// \param	id	-	the id of the sound to use
///	\return	The volume level at the outside angle of the cone
///
extern long MediaGetConeOutsideVolume( unsigned int id );

/// \brief	This function retrieves the position in 3D space, of the sound location
/// \param	id	-	the id of the sound to use
/// \param	position	-	this will be filled with a 3D coordinate of the sound position
///
extern void MediaGetSoundPosition( unsigned int id, float position[] );

/// \brief	This gets the speed vector of the sound (for the doppler effect)
/// \param	id	-	the id of the sound to use
/// \param	velocity	-	this will be filled with a 3D vector 
///							representing the speed of the sound.
///
extern void MediaGetSoundVelocity( unsigned int id, float velocity[] );

/// \brief	Gets the maximum distance (in meters) at which the sound can be heard
/// \param	id	-	the id of the sound to use
///	\return	the maximum distance at which the sound can be heard 
///
extern float MediaGetMaxDistance( unsigned int id );

/// \brief	Gets the minimum distance (in meters) of this sound
/// \param	id	-	the id of the sound to use
///	\return	the minimum distance of the sound 
///
extern float MediaGetMinDistance( unsigned int id );




#endif



